jQuery(document).ready(function ($) {
  'use strict';

  const FIELD_NAME = 'wil-single-product';
  const CONTROL_NAME = WilokeRestaurantMenu.prefix + FIELD_NAME;

  async function createSelect2($el, initialData = [], action) {
    $el.select2({
      data: initialData,
      ajax: {
        url: ajaxurl,
        dataType: 'json',
        data(params) {
          return {
            q: params.term, // search term
            page: params.page,
            action,
          };
        },
        processResults(data, params) {
          params.page = params.page || 1;
          return {
            results: data,
            pagination: {
              more: params.page * 30 < data.length,
            },
          };
        },
      },
    });
  }

  function getSelect2Value($el) {
    return $el.select2('data').map(({ id, text }) => ({ id, text, selected: true }));
  }

  const productControl = elementor.modules.controls.BaseData.extend({
    onReady() {
      this.registerDomSelector();
      this.fieldInitial();
      this.bindFieldChange();
      this.setStyles();
    },

    onBeforeDestroy() {
      if (this.$categories && this.$categories.select2) {
        this.$categories.select2('destroy');
      }
      if (this.$categories && this.$tags.select2) {
        this.$tags.select2('destroy');
      }
    },

    setStyles() {
      const styles = `
            .wil-width-100 {
                width:100%;
            }
            .wil-mb-15 {
                margin-bottom: 15px !important;
            }
          `;
      const id = 'wil-internal-style';
      const el = document.getElementById(id);
      if (el) {
        return;
      }
      const style = document.createElement('style');
      style.id = id;
      style.innerHTML = styles;
      document.head.appendChild(style);
    },

    registerDomSelector() {
      this.$products = this.$el.find('.wil-select-product');
      this.$fieldGroup = this.$el.find('.wil-field-group');
    },

    fieldInitial() {
      const settings = this.getCurrentValue();
      this.productsSelect(settings.products);
    },

    handleProductFilter() {
      this.saveValue();
    },

    bindFieldChange() {
      this.$products.on('change', this.saveValue.bind(this));
    },

    saveValue() {
      this.setValue({
        products: getSelect2Value(this.$products),
        FIELD_NAME,
      });
    },

    productsSelect(initialData = []) {
      createSelect2(this.$products, initialData, WilokeRestaurantMenu.prefix + '_custom_wil_product');
    },
  });
  elementor.addControlView(CONTROL_NAME, productControl);
});